#include "list.h"

/* Takes the pointer to the head of a linked list and a string
 * Creates a new node, inserts the string into that node, and 
 * points the last node in the list to it.
 * No return (void)
 */
node* insert(node* head, char* word) {
	node* newNode = malloc(sizeof(node));
	if(newNode == NULL) { // malloc() failed to reserve space
		printf("Error: Could not reserve memory space for node.\n");
		return NULL; // exit function
	}

	newNode->word = malloc(sizeof(char)*MAX_WORD_LEN); // Give this node memory space for the word
	strcpy(newNode->word, word); // Copy the string from the given temporary char array to its new permanent location in the struct

	if(head == NULL) {
		head->word = malloc(sizeof(char)*MAX_WORD_LEN);
		strcpy(head->word, word);
	} else {
		node* currentNode = head;
		while(currentNode->next != NULL) {
			currentNode = currentNode->next;
		}
		currentNode->next = newNode;
	}
	return head;

/*	if(head == NULL) { // Starting with an empty list
		head = newNode;
	} else {
		node* currentNode = head;
		while(currentNode->next != NULL) {
			currentNode = currentNode->next;
		}
		currentNode->next = newNode;
	}
	return head;*/
}

/* Takes an input filename, and opens that file for reading,
 * then loads every word in the file into a linked list.
 * Return on failure: NULL
 * Return on success: pointer to the head of the list
 */
node* create_list(char* filename) {
	FILE* input = fopen(filename, "r");
	if(input == NULL) {
		return NULL; // Return NULL for error
	}

/*	node* head = malloc(sizeof(node)); // Create an initial head pointer
	head->word = malloc(sizeof(char) * MAX_WORD_LEN);
	head->next = NULL; */
	node* head = malloc(sizeof(node));

	char* word = malloc(sizeof(char) * MAX_WORD_LEN);
	while(fscanf(input, "%s\n", word) != EOF) {
		head = insert(head, word);
		if(head == NULL) { // insert_at_head() returned NULL for error
			return NULL; // Return NULL for error
		}
	}

	fclose(input);
	return head; // Return the head of the list
}

void print_list(node* list) {
	node* tmp = list;
	while(tmp != NULL) {
		printf("%s --> ", tmp->word);
		tmp = tmp->next;
	}
	printf("\n");
}
